@@ -7,7 +7,7 @@ Attean::API::Iterator - Typed iterator
 
 =head1 VERSION
 
-This document describes Attean::API::Iterator version 0.011
+This document describes Attean::API::Iterator version 0.012
 
 =head1 DESCRIPTION
 
@@ -94,7 +94,7 @@ elements from the referent.
 
 =cut
 
-package Attean::API::Iterator 0.011 {
+package Attean::API::Iterator 0.012 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str Object InstanceOf);
@@ -189,7 +189,7 @@ package Attean::API::Iterator 0.011 {
 		}
 		
 		# copy variables into new iterator if $self does ::ResultIterator or ::ResultOrTermIterator
-		my %args;
+		my %args	= @_;
 		if ($self->can('variables')) {
 			$args{variables}	= $self->variables;
 		}
@@ -256,7 +256,7 @@ Print each item as it is consumed (with the string generated by C<< as_string >>
 	}
 }
 
-package Attean::API::RepeatableIterator 0.011 {
+package Attean::API::RepeatableIterator 0.012 {
 	use Moo::Role;
 	requires 'reset';
 	
@@ -292,7 +292,7 @@ package Attean::API::CanonicalizingBindingIterator {
 	}
 }
 
-package Attean::API::ResultOrTermIterator 0.011 {
+package Attean::API::ResultOrTermIterator 0.012 {
 	use Moo::Role;
 	use Types::Standard qw(ArrayRef Str);
 	has 'variables' => (is => 'rw', isa => ArrayRef[Str], default => sub { [] });
@@ -330,7 +330,7 @@ package Attean::API::ResultOrTermIterator 0.011 {
 	};
 }
 
-package Attean::API::TripleIterator 0.011 {
+package Attean::API::TripleIterator 0.012 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 	sub as_quads {
@@ -340,12 +340,12 @@ package Attean::API::TripleIterator 0.011 {
 	}
 }
 
-package Attean::API::QuadIterator 0.011 {
+package Attean::API::QuadIterator 0.012 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 }
 
-package Attean::API::MixedStatementIterator 0.011 {
+package Attean::API::MixedStatementIterator 0.012 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 	sub as_quads {
@@ -358,7 +358,7 @@ package Attean::API::MixedStatementIterator 0.011 {
 	}
 }
 
-package Attean::API::ResultIterator 0.011 {
+package Attean::API::ResultIterator 0.012 {
 	use Moo::Role;
 	use Types::Standard qw(Str ArrayRef);
 	with 'Attean::API::CanonicalizingBindingIterator';
@@ -379,7 +379,7 @@ package Attean::API::ResultIterator 0.011 {
 	}
 }
 
-package Attean::API::TermIterator 0.011 {
+package Attean::API::TermIterator 0.012 {
 	use Moo::Role;
 	sub canonicalize {
 		my $self	= shift;
